#include "stdafx.h"
#include "OSMCtrlApp.h"
#include "MainFrm.h"
#include "OSMCtrlAppDoc.h"
#include "OSMCtrlAppView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG


#pragma warning(suppress: 26409 26433 26440 26477)
IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd) //NOLINT(modernize-avoid-c-arrays)
  ON_WM_CREATE()
  ON_UPDATE_COMMAND_UI(ID_INDICATOR_POSITION, &CMainFrame::OnUpdatePosition)
  ON_UPDATE_COMMAND_UI(ID_INDICATOR_LENGTH, &CMainFrame::OnUpdateLength)
END_MESSAGE_MAP()

std::array<UINT, 3> indicators{ { ID_SEPARATOR, ID_INDICATOR_POSITION, ID_INDICATOR_LENGTH } };

#pragma warning(suppress: 26434)
int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
  //Let the base class do its thing
  if (__super::OnCreate(lpCreateStruct) == -1)
    return -1;

  //Create the toolbar
  if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP | CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
      !m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
  {
    TRACE(_T("Failed to create toolbar\n"));
    return -1;
  }

  //Create the status bar
#pragma warning(suppress: 26472)
  if (!m_wndStatusBar.Create(this) || !m_wndStatusBar.SetIndicators(indicators.data(), static_cast<int>(indicators.size())))
  {
    TRACE(_T("Failed to create status bar\n"));
    return -1;
  }

  m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
  EnableDocking(CBRS_ALIGN_ANY);
  DockControlBar(&m_wndToolBar);

  return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
  cs.style &= ~FWS_ADDTOTITLE; //Do not add the document name to the caption
  return __super::PreCreateWindow(cs);
}

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
  __super::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
  __super::Dump(dc);
}
#endif //#ifdef _DEBUG

#pragma warning(suppress: 26429)
void CMainFrame::OnUpdatePosition(CCmdUI* pCmdUI)
{
  //Update various status bar indicators
#pragma warning(suppress: 26466)
  auto pView{static_cast<COSMCtrlAppView*>(GetActiveFrame()->GetActiveView())};

  //only do it if the OSM Ctrl view is active
  bool bViewValid{false};
  if (pView)
    bViewValid = pView->IsKindOf(RUNTIME_CLASS(COSMCtrlAppView));

  if (bViewValid)
  {
    //Get the client rect
    CRect rClient;
    pView->m_ctrlOSM.GetClientRect(&rClient);

    //Get the cursor position in view client coordinates
    CPoint ptCursor;
    GetCursorPos(&ptCursor);
    pView->ScreenToClient(&ptCursor);
  #ifdef COSMCTRL_NOD2D
    const Gdiplus::PointF tempCursor{static_cast<Gdiplus::REAL>(ptCursor.x), static_cast<Gdiplus::REAL>(ptCursor.y)};
  #else
    const CD2DPointF tempCursor{static_cast<FLOAT>(ptCursor.x), static_cast<FLOAT>(ptCursor.y)};
  #endif //#ifdef COSMCTRL_NOD2D

    COSMCtrlPosition position;
    if (pView->GetCtrl().ClientToPosition(tempCursor, rClient, position))
    {
      //Calculate the position in DMS format
      CString sPosition;
      const double fAbsLatitude{fabs(position.m_fLatitude)};
      const int nLatitudeD{static_cast<int>(fAbsLatitude)};
      const int nLatitudeM{static_cast<int>((fAbsLatitude - nLatitudeD) * 60)};
      const int nLatitudeS{static_cast<int>((fAbsLatitude - nLatitudeD - nLatitudeM/60.0) * 3600)};
#pragma warning(suppress: 26485)
      const TCHAR* pszLatitudeType{(position.m_fLatitude > 0) ? _T("N") : _T("S")};
      const double fAbsLongitude{fabs(position.m_fLongitude)};
      const int nLongitudeD{static_cast<int>(fAbsLongitude)};
      const int nLongitudeM{static_cast<int>((fAbsLongitude - nLongitudeD) * 60)};
      const int nLongitudeS{static_cast<int>((fAbsLongitude - nLongitudeD - nLongitudeM/60.0) * 3600)};
#pragma warning(suppress: 26485)
      const TCHAR* pszLongitudeType{(position.m_fLongitude > 0) ? _T("E") : _T("W")};

      sPosition.Format(_T("Position: %d %d' %d\" %s  %d %d' %d\" %s"), nLatitudeD, nLatitudeM, nLatitudeS, pszLatitudeType, nLongitudeD, nLongitudeM, nLongitudeS, pszLongitudeType); 
      m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_POSITION), sPosition);
    }
    else
      m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_POSITION), _T("Position: N/A"));
  }
  else
    m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_POSITION), _T("Position: N/A"));
#pragma warning(suppress: 26486)
  pCmdUI->Enable(TRUE);
}


#pragma warning(suppress: 26429)
void CMainFrame::OnUpdateLength(CCmdUI* pCmdUI)
{
  //Update various status bar indicators
#pragma warning(suppress: 26466)
  auto pView{static_cast<COSMCtrlAppView*>(GetActiveFrame()->GetActiveView())};

  //only do it if the OSM Ctrl view is active
  bool bViewValid{false};
  if (pView)
    bViewValid = pView->IsKindOf(RUNTIME_CLASS(COSMCtrlAppView));

  if (bViewValid)
  {
    bool bFoundItem{false};
    double dDistance{0};
    double dBearing{0};
    bool bValidBearing{false};
    const INT_PTR nPolyline{pView->GetCtrl().GetFirstSelectedPolyline()};
    if (nPolyline != -1)
    {
#pragma warning(suppress: 26446)
      const auto& polyline{pView->GetCtrl().m_Polylines[nPolyline]};
      dDistance = polyline->Length() / 1000; //We want the distance in KM not meters
      dBearing = polyline->LastBearing();
      bFoundItem = true;
      bValidBearing = true;
    }
    else
    {
      const INT_PTR nPolygon{pView->GetCtrl().GetFirstSelectedPolygon()};
      if (nPolygon != -1)
      {
#pragma warning(suppress: 26446)
        const auto& polygon{pView->GetCtrl().m_Polygons[nPolygon]};
        dDistance = polygon->Length() / 1000; //We want the distance in KM not meters 
        bFoundItem = true;
      }
    }

    if (bFoundItem)
    {
      //Determine if we should use the Metric or Imperial system of measurement
      bool bMetric{true};
      switch (pView->GetCtrl().GetScaleBarUnits())
      {
        case COSMCtrl::ScaleBarUnits::UseOSDefault:
        {
          CString sMeasure;
          const int nGLI{GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_IMEASURE, sMeasure.GetBuffer(3), 3)};
          sMeasure.ReleaseBuffer();
          if (nGLI)
          {
            if (sMeasure == _T("1"))
            {
              //Convert the measured distance to miles
              bMetric = false;
            }
          }
          break;
        }
        case COSMCtrl::ScaleBarUnits::Metric:
        {
          bMetric = true;
          break;
        }
        case COSMCtrl::ScaleBarUnits::Imperial:
        {
          bMetric = false;
          break;
        }
        default:
        {
          ASSERT(false);
          break;
        }
      }
      
      //Convert the distance to miles if necessary
      if (!bMetric)
        dDistance *= 0.621371192;

      //Calculate the length part of the string
      CString sDistance;
      if (bMetric)
      {
        if (dDistance < 1)
          sDistance.Format(_T("%.0f Meters"), dDistance*1000);
        else
          sDistance.Format(_T("%.3f Kilometers"), dDistance);
      }
      else
        sDistance.Format(_T("%.3f Miles"), dDistance);

      CString sDetails;
      if (bValidBearing)
      {
        const int nBearingD{static_cast<int>(dBearing)};
        const int nBearingM{static_cast<int>((dBearing - nBearingD) * 60)};
        sDetails.Format(_T("Length: %s, Bearing: %d %d'"), sDistance.GetString(), nBearingD, nBearingM);
      }
      else
        sDetails.Format(_T("Length: %s, Bearing: N/A"), sDistance.GetString());

      m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_LENGTH), sDetails);
    }
    else
      m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_LENGTH), _T("Length: N/A, Bearing: N/A"));
  }
  else
    m_wndStatusBar.SetPaneText(m_wndStatusBar.CommandToIndex(ID_INDICATOR_LENGTH), _T("Length: N/A, Bearing: N/A"));
#pragma warning(suppress: 26486)
  pCmdUI->Enable(TRUE);
}
