/*
Module : OSMCtrlCircle.h
Purpose: Defines the interface for COSMCtrlCircle class
Created: PJN / 10-04-2011

Copyright (c) 2011 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Includes ////////////////////////////////////////

#pragma once

#ifndef __OSMCTRLCIRCLE_H__
#define __OSMCTRLCIRCLE_H__

#include "OSMCtrlPosition.h"
#include "OSMCtrlHelper.h"


//////////////////// Classes //////////////////////////////////////////////////

class COSMCtrl;

//A class which represents a circle (i.e. a closed sequence of nodes with a specific radius) on the map
class COSMCtrlCircle
{
public:
//Constructors / Destructors
  COSMCtrlCircle() = default;
  COSMCtrlCircle(const COSMCtrlCircle&) = default;
  COSMCtrlCircle(COSMCtrlCircle&&) noexcept = default;
  virtual ~COSMCtrlCircle() = default;

//Enums
  enum class DrawingStyle
  {
    LineOnly,
    LineAndInside,
  };

//Methods
  COSMCtrlCircle& operator=(const COSMCtrlCircle&) = default;
#pragma warning(suppress: 26439)
  COSMCtrlCircle& operator=(COSMCtrlCircle&&) = default;
  _Return_type_success_(return != 0) bool GetBoundingRect(_Out_ COSMCtrlPosition& topLeft, _Out_ COSMCtrlPosition& bottomRight) const noexcept;
#ifdef COSMCTRL_NOD2D
  virtual bool Draw(_In_ const COSMCtrl& ctrl, _In_ Gdiplus::GraphicsPath& path, _In_ bool bCircumference, _In_ const CRect& rClient) const;
  virtual bool Draw(_In_ COSMCtrl& ctrl, _In_ Gdiplus::Graphics& graphics, _In_ const CRect& rClient);
#else
  virtual bool Draw(_In_ COSMCtrl& ctrl, _In_ CRenderTarget* pRenderTarget, _In_ const CRect& rClient);
#endif //#ifdef COSMCTRL_NOD2D

//Member variables
  COSMCtrlPosition m_Position; //The position of this circle
  double m_fRadius{0}; //The radius distance in meters
  bool m_bSelected{false}; //Is this circle selected
  float m_fLinePenWidth{2}; //The width of the line to draw
#ifdef COSMCTRL_NOD2D
  Gdiplus::Color m_colorPen{0, 0, 255}; //The color of the pen to use
  Gdiplus::Color m_colorSelection{255, 0, 0}; //The color of the brush to use for selection
  Gdiplus::Color m_colorBrush{128, 0, 0, 255}; //The color of the brush to use
  Gdiplus::DashStyle m_DashStyle{Gdiplus::DashStyleSolid}; //The dash style to use
#else
  D2D1::ColorF m_colorPen{0, 0, 255}; //The color of the pen to use
  D2D1::ColorF m_colorSelection{255, 0, 0}; //The color of the brush to use for selection
  D2D1::ColorF m_colorBrush{0, 0, 255, 0.5F}; //The color of the brush to use
  D2D1_DASH_STYLE m_DashStyle{D2D1_DASH_STYLE_SOLID}; //The dash style to use
#endif //#ifdef COSMCTRL_NOD2D
  DrawingStyle m_DrawingStyle{DrawingStyle::LineAndInside}; //The drawing style to use for this circle
  CString m_sToolTipText; //The tooltip text for this circle
  bool m_bDraggable{false}; //Is this circle draggable
  bool m_bVisible{true}; //Is this circle currently visible
  bool m_bEditable{false}; //Is this circle editable
  bool m_bHitTest{true}; //Is this circle to be hit-tested
  int m_nMinZoomLevel{COSMCtrlHelper::OSMMinZoom}; //The minimum zoom level at which this circle appears
  int m_nMaxZoomLevel{COSMCtrlHelper::OSMMaxZoom}; //The max zoom level at which this circle appears
};


#endif //#ifndef __OSMCTRLCIRCLE_H__
