/*
Module : OSMCtrlMarker.h
Purpose: Defines the interface for COSMCtrlMarker class
Created: PJN / 10-04-2011

Copyright (c) 2011 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Includes ////////////////////////////////////////

#pragma once

#ifndef __OSMCTRLMARKER_H__
#define __OSMCTRLMARKER_H__

#include "OSMCtrlPosition.h"
#include "OSMCtrlHelper.h"


//////////////////// Classes //////////////////////////////////////////////////

class COSMCtrl;

//A class which represents a marker which is a icon at a specific position on the map
class COSMCtrlMarker
{
public:
//Constructors / Destructors
  COSMCtrlMarker() = default;
  COSMCtrlMarker(const COSMCtrlMarker&) = default;
#pragma warning(suppress: 26439)
  COSMCtrlMarker(COSMCtrlMarker&&) = default;
  virtual ~COSMCtrlMarker() = default;

//Methods
  COSMCtrlMarker& operator=(const COSMCtrlMarker&) = default;
#pragma warning(suppress: 26439)
  COSMCtrlMarker& operator=(COSMCtrlMarker&&) = default;

#ifdef COSMCTRL_NOD2D
  virtual bool Draw(_In_ COSMCtrl& ctrl, _In_ Gdiplus::Graphics& graphics, _In_ const CRect& rClient);
#else
  virtual bool DrawInternal(_In_ COSMCtrl& ctrl, _In_ CRenderTarget* pRenderTarget, _In_ CD2DBitmap* pBitmap, _In_ CD2DPointF& ptMarker);
  virtual bool Draw(_In_ COSMCtrl& ctrl, _In_ CRenderTarget* pRenderTarget, _In_ const CRect& rClient);
#endif //#ifdef COSMCTRL_NOD2D

//Member variables
  INT_PTR m_nIconIndex{-1}; //The index of the icon for this marker (Using this level of indirection means that we keep down memory usage so that multiple markers can reuse the same icon resource
  COSMCtrlPosition m_Position; //The position of this marker
  bool m_bSelected{false}; //Is this marker selected
  CString m_sToolTipText; //The tooltip text for this marker
  bool m_bDraggable{false}; //Is this marker draggable
  bool m_bVisible{true}; //Is this marker currently visible
  bool m_bHitTest{true}; //Is this marker to be hit-tested
  int m_nMinZoomLevel{COSMCtrlHelper::Misc::OSMMinZoom}; //The minimum zoom level at which this marker appears
  int m_nMaxZoomLevel{COSMCtrlHelper::Misc::OSMMaxZoom}; //The max zoom level at which this marker appears
#ifdef COSMCTRL_NOD2D
  Gdiplus::Color m_colorSelection{255, 255, 0, 0}; //The color of the brush to use for selection
#else
  D2D1::ColorF m_colorSelection{255, 0, 0, 1.0F}; //The color of the brush to use for selection
#endif //#ifdef COSMCTRL_NOD2D
  float m_fSelectionPenWidth{2}; //The width of the selection rectangle to draw
};


#endif //#ifndef __OSMCTRLMARKER_H__
