/*
Module : OSMCtrlNode.h
Purpose: Defines the interface for COSMCtrlNode class
Created: PJN / 10-04-2011

Copyright (c) 2011 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Includes ////////////////////////////////////////

#pragma once

#ifndef __OSMCTRLNODE_H__
#define __OSMCTRLNODE_H__

#include "OSMCtrlPosition.h"


//////////////////// Classes //////////////////////////////////////////////////

//A class which represents a node on the map (a combination of COSMCtrlPosition and a selected state)
class COSMCtrlNode
{
public:
//Constructors / Destructors
  COSMCtrlNode() = default;
  COSMCtrlNode(_In_ double fLongitude, _In_ double fLatitude, _In_ double fBearing = 0, _In_ double fSpeed = 0, _In_ bool bSelected = false, _In_ bool bBearingValid = false, _In_ bool bSpeedValid = false, _In_ double fHDOP = 0) noexcept : m_Position(fLongitude, fLatitude, fBearing, fSpeed, bBearingValid, bSpeedValid, fHDOP),
                                                                                                                                                                                                                                               m_bSelected(bSelected)
  {
  }
  COSMCtrlNode(const COSMCtrlNode&) = default;
  COSMCtrlNode(COSMCtrlNode&&) noexcept = default;
  ~COSMCtrlNode() = default;

//Methods
  COSMCtrlNode& operator=(const COSMCtrlNode&) = default;
  COSMCtrlNode& operator=(COSMCtrlNode&&) = default;

//Member variables
  COSMCtrlPosition m_Position;
  bool m_bSelected{false};
};


#endif //#ifndef __OSMCTRLNODE_H__
