/*
Module : OSMCtrlPosition.h
Purpose: Defines the interface for COSMCtrlPosition class
Created: PJN / 10-04-2011

Copyright (c) 2011 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Includes ////////////////////////////////////////

#pragma once

#ifndef __OSMCTRLPOSITION_H__
#define __OSMCTRLPOSITION_H__


//////////////////// Classes //////////////////////////////////////////////////

//A class which represents a position on the map (i.e. a lat / long set of coordinates)
class COSMCtrlPosition
{
public:
//Constructors / Destructors
  COSMCtrlPosition() = default;
  COSMCtrlPosition(_In_ double fLongitude, _In_ double fLatitude, _In_ double fBearing = 0, _In_ double fSpeed = 0, _In_ bool bBearingValid = false, _In_ bool bSpeedValid = false, _In_ double fHDOP = 0) noexcept;
  COSMCtrlPosition(const COSMCtrlPosition&) = default;
  COSMCtrlPosition(COSMCtrlPosition&&) = default;
  ~COSMCtrlPosition() = default;

//Methods
  COSMCtrlPosition& operator=(const COSMCtrlPosition&) = default;
  COSMCtrlPosition& operator=(COSMCtrlPosition&&) = default;
  void NormalizeLongitude() noexcept;

//Member variables
  double m_fLongitude{0}; //The longitude of this marker
  double m_fLatitude{0}; //The latitude of this marker
  double m_fBearing{0}; //The compass bearing (0 - North, 90 - East etc)
  bool m_bBearingValid{false}; //Is the bearing value valid
  double m_fSpeed{0}; //The speed in knots
  bool m_bSpeedValid{false}; //Is the speed value valid
  double m_fHDOP{0}; //The horizontal dilution of precision value
};


#endif //#ifndef __OSMCTRLPOSITION_H__
