/*
Module : OSMCtrlTileProviders.h
Purpose: Defines the interface for an MFC GUI control which implements display of OpenStreetMaps tiles
Created: PJN / 28-11-2009

Copyright (c) 2011 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Includes ////////////////////////////////////////

#pragma once

#ifndef __OSMCTRLTILEPROVIDERS_H__
#define __OSMCTRLTILEPROVIDERS_H__


//////////////////// Classes //////////////////////////////////////////////////

//The tile provider base class
class IOSMCtrlTileProvider
{
public:
//Methods
  virtual CString GetDownloadURL(_In_ int nZoom, _In_ int nTileX, _In_ int nTileY) = 0;
  virtual CString GetDownloadServer() = 0;
  virtual CString GetDownloadObject(_In_ int nZoom, _In_ int nTileX, _In_ int nTileY) = 0;
  virtual INTERNET_PORT GetDownloadPort() = 0;
};


class COSMCtrlMapnikTileProvider : public IOSMCtrlTileProvider
{
public:
//Constructors / Destructors
  COSMCtrlMapnikTileProvider() = default;
  COSMCtrlMapnikTileProvider(const COSMCtrlMapnikTileProvider&) = delete;
  COSMCtrlMapnikTileProvider(COSMCtrlMapnikTileProvider&&) = delete;
  virtual ~COSMCtrlMapnikTileProvider() = default;

//Methods
  COSMCtrlMapnikTileProvider& operator=(const COSMCtrlMapnikTileProvider&) = delete;
  COSMCtrlMapnikTileProvider& operator=(COSMCtrlMapnikTileProvider&&) = delete;

  CString GetDownloadURL(_In_ int nZoom, _In_ int nTileX, _In_ int nTileY) override
  {
    CString sURL;
    sURL.Format(_T("http://%s/%s"), GetDownloadServer().GetString(), GetDownloadObject(nZoom, nTileX, nTileY).GetString());
    return sURL;
  }

  CString GetDownloadServer() override
  {
    return CString(_T("tile.openstreetmap.org"));
  }

  CString GetDownloadObject(_In_ int nZoom, _In_ int nTileX, _In_ int nTileY) override
  {
    CString sObject;
    sObject.Format(_T("/%d/%d/%d.png"), nZoom, nTileX, nTileY);
    return sObject;
  }

  INTERNET_PORT GetDownloadPort() noexcept override
  {
    return 80;
  }
};


class COSMCtrlCyclemapTileProvider : public IOSMCtrlTileProvider
{
public:
//Constructors / Destructors
  COSMCtrlCyclemapTileProvider() = default;
  COSMCtrlCyclemapTileProvider(const COSMCtrlCyclemapTileProvider&) = delete;
  COSMCtrlCyclemapTileProvider(COSMCtrlCyclemapTileProvider&&) = delete;
  virtual ~COSMCtrlCyclemapTileProvider() = default;

//Methods
  COSMCtrlCyclemapTileProvider& operator=(const COSMCtrlCyclemapTileProvider&) = delete;
  COSMCtrlCyclemapTileProvider& operator=(COSMCtrlCyclemapTileProvider&&) = delete;

  CString GetDownloadURL(_In_ int nZoom, _In_ int nTileX, _In_ int nTileY) override
  {
    CString sURL;
    sURL.Format(_T("http://%s/%s"), GetDownloadServer().GetString(), GetDownloadObject(nZoom, nTileX, nTileY).GetString());
    return sURL;
  }

  CString GetDownloadServer() override
  {
    return CString(_T("tile.opencyclemap.org"));
  }

  CString GetDownloadObject(_In_ int nZoom, _In_ int nTileX, _In_ int nTileY) override
  {
    CString sObject;
    sObject.Format(_T("/cycle/%d/%d/%d.png"), nZoom, nTileX, nTileY);
    return sObject;
  }

  INTERNET_PORT GetDownloadPort() noexcept override
  {
    return 80;
  }
};


#endif //#ifndef __OSMCTRLTILEPROVIDERS_H__
